{% extends "groups/comment_base.html" %}
{% load static %}

{% block comment_visible %}
    <p>
        Uploaded a file:
        <a href="{% get_media_prefix %}{{ comment.file }}">{{ comment.short_filename }}</a>
    </p>
{% endblock comment_visible %}
