{% extends 'base.html' %}

{% block title %}
    {% block groups_title %}{% endblock groups_title %}
{% endblock title %}

{% block subtitle %}
    {% block groups_subtitle %}{% endblock groups_subtitle %}
{% endblock subtitle %}

{% block main_content %}
    {% block back_link %}{% endblock back_link %}
    {% block groups_main_content %}{% endblock groups_main_content %}
{% endblock main_content %}
