{% extends "groups/base.html" %}

{% load crispy_forms_tags %}

{% load groups_tags %}

{% block groups_title %}Comments on {{ discussion.name }}{% endblock groups_title %}

{% block groups_subtitle %}
    <h2>Comments on {{ discussion.name }}</h2>
{% endblock groups_subtitle %}

{% block back_link %}
    <p><a href="{% url 'group-detail' pk=group.pk %}">Back to {{ group.name }}</a></p>
{% endblock back_link %}

{% block groups_main_content %}
    {% include "groups/subscribe_button.html" %}
    <ul>
        {% for comment in comments %}
            <li id="{{ comment.get_pagejump_anchor }}">
                {% block comment %}
                    {% comment_render comment request %}
                {% endblock comment %}
            </li>
        {% endfor %}
    </ul>

    {% crispy form %}

    <p><a href="{% url 'comment-upload-file' pk=discussion.pk %}">Upload a file to this discussion</a></p>
{% endblock groups_main_content %}
